package com.ruoyi.project.VRdemo.controller;

import com.alibaba.excel.EasyExcel;
import com.ruoyi.common.utils.poi.*;
import com.ruoyi.framework.aspectj.lang.annotation.Anonymous;
import com.ruoyi.framework.web.controller.BaseController;
import com.ruoyi.framework.web.domain.AjaxResult;
import com.ruoyi.project.VRdemo.domain.*;
import com.ruoyi.project.VRdemo.mapper.DataExcelMapper;
import com.ruoyi.project.VRdemo.service.INoticeDataService;
import com.ruoyi.project.VRdemo.service.IPressService;
import com.ruoyi.project.VRdemo.service.IStudyDataService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * excel文件的导入和导出
 */
@RestController
@RequestMapping("/excel")
public class DataIoController extends BaseController {
    @Autowired
    private DataExcelMapper DataExcelMapper;

//    @Autowired
//    private IAppService appService;

    @Autowired
    private INoticeDataService noticeDataService;

    @Autowired
    private IStudyDataService studyDataService;
    @Autowired
    private IPressService pressService;
    //        应用导出数据
    @PreAuthorize("@ss.hasAnyRoles('admin,company')")
    @GetMapping("/export/app")
    public void exportData(HttpServletResponse response) throws IOException {
        // 查询要导出的数据
        List<ExportApp> dataList = DataExcelMapper.getAppData();

        // 设置导出的文件名和响应头
        String fileName = "exportData.xlsx";
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        response.setHeader("Content-Disposition", "attachment; filename=" + URLEncoder.encode(fileName, "UTF-8"));

        // 创建EasyExcel写出器
        EasyExcel.write(response.getOutputStream(), ExportApp.class).registerWriteHandler(new CustomCellWriteWeightConfig()).sheet("Sheet1").doWrite(dataList);
    }

    //        应用导入数据
//    @PreAuthorize("@ss.hasAnyRoles('admin,company')")
//    @PostMapping("/import/app")
//    public AjaxResult importData(MultipartFile file, boolean updateSupport) throws IOException {
//        AppImportListener appimportListener = new AppImportListener(DataExcelMapper);
//        // 创建EasyExcel读取器4
//        EasyExcel.read(file.getInputStream(), ImportApp.class, appimportListener).sheet().headRowNumber(1).doRead();
//
//        String message = appService.importUser(appimportListener.getAppDataList(), updateSupport);
//        return success(message);
//    }

    //      应用下载模板
    @PreAuthorize("@ss.hasAnyRoles('admin,company')")
    @PostMapping("/importTemplate/app")
    public void importTemplate(HttpServletResponse response) throws IOException {
        List<ImportApp> dataList = new ArrayList<>();
        EasyExcel.write(response.getOutputStream(), ImportApp.class).sheet("Sheet1").doWrite(dataList);
    }

    //        活动预告导出数据
    @PreAuthorize("@ss.hasAnyRoles('admin,company,xmg')")
    @GetMapping("/export/noticeData")
    public void exNoticeData(HttpServletResponse response) throws IOException {
        // 查询要导出的数据
        List<NoticeExData> dataList = DataExcelMapper.getNoticeData();

        // 遍历数据列表，将持有类型字段从数字转换为标签
        for (NoticeExData data : dataList) {
            Integer holdingTypeValue = Integer.parseInt(data.getHoldingType()); // 将字符串转换为整数
            String holdingTypeLabel = DataExcelMapper.getHoldingTypeLabel(holdingTypeValue); // 调用标签转换方法
            data.setHoldingType(holdingTypeLabel); // 设置为中文标签
        }


        // 设置导出的文件名和响应头
        String fileName = "exportData.xlsx";
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        response.setHeader("Content-Disposition", "attachment; filename=" + URLEncoder.encode(fileName, "UTF-8"));

        // 创建EasyExcel写出器
        EasyExcel.write(response.getOutputStream(), NoticeExData.class).registerWriteHandler(new CustomCellWriteWeightConfig()).sheet("Sheet1").doWrite(dataList);
    }


    //        活动预告导入数据
    @PreAuthorize("@ss.hasAnyRoles('admin,company,xmg')")
    @PostMapping("/import/notice")
    public AjaxResult imNoticeData(MultipartFile file, boolean updateSupport) throws IOException {

        NoticeImListener noticeImListener = new NoticeImListener(DataExcelMapper);
        // 创建EasyExcel读取器4
        EasyExcel.read(file.getInputStream(), NoticeImData.class, noticeImListener).sheet().headRowNumber(1).doRead();

        String message = noticeDataService.importNotice(noticeImListener.getDataList(), updateSupport);
        return success(message);
    }

    //      活动预告下载模板
    @PreAuthorize("@ss.hasAnyRoles('admin,company,xmg')")
    @PostMapping("/importTemplate/notice")
    public void imNoticeTemplate(HttpServletResponse response) throws IOException {
        List<NoticeImData> dataList = new ArrayList<>();
        EasyExcel.write(response.getOutputStream(), NoticeImData.class).sheet("Sheet1").doWrite(dataList);
    }

    //        研究导出数据
    @PreAuthorize("@ss.hasAnyRoles('admin,company,xmg')")
    @GetMapping("/export/studydata")
    public void exStudyData(HttpServletResponse response) throws IOException {
        // 查询要导出的数据
        List<StudyExData> dataList = DataExcelMapper.getStudyData();

        // 获取年份字典数据和月份字典数据
        List<Map<String, String>> yearDictDataList = DataExcelMapper.getDictDataByType("year_type");
        List<Map<String, String>> monthDictDataList = DataExcelMapper.getDictDataByType("month_type");

        // 创建字典数据的映射，以便后续转换字段值
        Map<String, String> yearDictMap = yearDictDataList.stream()
                .collect(Collectors.toMap(map -> map.get("dict_value"), map -> map.get("dict_label")));

        Map<String, String> monthDictMap = monthDictDataList.stream()
                .collect(Collectors.toMap(map -> map.get("dict_value"), map -> map.get("dict_label")));

        // 处理字典字段，将字段值转换为标签
        for (StudyExData studyData : dataList) {
            String yearValue = studyData.getYear();
            String yearLabel = yearDictMap.getOrDefault(yearValue, "");

            String monthValue = studyData.getMonth();
            String monthLabel = monthDictMap.getOrDefault(monthValue, "");

            studyData.setYear(yearLabel);
            studyData.setMonth(monthLabel);
        }

        // 设置导出的文件名和响应头
        String fileName = "exportData.xlsx";
        response.setContentType("application/vnd.ms-excel");
        response.setCharacterEncoding("utf-8");
        response.setHeader("Content-Disposition", "attachment; filename=" + URLEncoder.encode(fileName, "UTF-8"));

        // 创建EasyExcel写出器
        EasyExcel.write(response.getOutputStream(), StudyExData.class).registerWriteHandler(new CustomCellWriteWeightConfig()).sheet("Sheet1").doWrite(dataList);
    }

    //        研究导入数据
    @PreAuthorize("@ss.hasAnyRoles('admin,company,xmg')")
    @PostMapping("/import/study")
    public AjaxResult imStudyData(MultipartFile file, boolean updateSupport) throws IOException {

        StudyImListener studyImListener = new StudyImListener(DataExcelMapper);
        // 创建EasyExcel读取器4
        EasyExcel.read(file.getInputStream(), StudyImData.class, studyImListener).sheet().headRowNumber(1).doRead();

        String message = studyDataService.importStudy(studyImListener.getDataList(), updateSupport);
        return success(message);
    }

    //     研究下载模板
    @PreAuthorize("@ss.hasAnyRoles('admin,company,xmg')")
    @PostMapping("/importTemplate/study")
    public void imStudyTemplate(HttpServletResponse response) throws IOException {
        List<StudyImData> dataList = new ArrayList<>();
        EasyExcel.write(response.getOutputStream(), StudyImData.class).sheet("Sheet1").doWrite(dataList);
    }


    /**
     * 导入press数据
     */
    @PreAuthorize("@ss.hasAnyRoles('admin,company')")
    @PostMapping("/import/press")
    public AjaxResult importPressData(MultipartFile file, boolean updateSupport) throws IOException {

        PressImportListener dataImportListener = new PressImportListener(DataExcelMapper);
        // 创建EasyExcel读取器4
        EasyExcel.read(file.getInputStream(), Press.class, dataImportListener).sheet().headRowNumber(1).doRead();
        String message = pressService.importPress(dataImportListener.getDataList(), updateSupport);
        return success(message);
    }
}
